"""
    FAST - Copyright (c) 2016 ONERA ISAE
"""
#  This file is part of FAST : A framework for rapid Overall Aircraft Design
#  Copyright (C) 2020  ONERA & ISAE-SUPAERO
#  FAST is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <https://www.gnu.org/licenses/>.

import numpy as np
from fastoad.models.aerodynamics.constants import POLAR_POINT_COUNT
from openmdao.core.explicitcomponent import ExplicitComponent


class ComputePolar(ExplicitComponent):
    def initialize(self):
        self.options.declare("low_speed_aero", default=False, types=bool)

    def setup(self):
        self.low_speed_aero = self.options["low_speed_aero"]

        self.add_input("tuning:aerodynamics:aircraft:cruise:CD:k", val=np.nan)
        self.add_input("tuning:aerodynamics:aircraft:cruise:CD:offset", val=np.nan)
        self.add_input("tuning:aerodynamics:aircraft:cruise:CD:winglet_effect:k", val=np.nan)
        self.add_input("tuning:aerodynamics:aircraft:cruise:CD:winglet_effect:offset", val=np.nan)

        nans_array = np.full(POLAR_POINT_COUNT, np.nan)
        if self.low_speed_aero:
            self.add_input("cl_low_speed", val=nans_array)
            self.add_input("cd0_total_low_speed", val=nans_array)
            self.add_input("cd_trim_low_speed", val=nans_array)
            self.add_input("cd_comp_low_speed", val=nans_array)
            self.add_input("oswald_coeff_low_speed", val=np.nan)

            self.add_output("aerodynamics:Cd_low_speed", shape=POLAR_POINT_COUNT)
        else:
            self.add_input("data:aerodynamics:aircraft:cruise:CL", val=nans_array)
            self.add_input("data:aerodynamics:aircraft:cruise:CD0", val=nans_array)
            self.add_input("data:aerodynamics:aircraft:cruise:CD:trim", val=nans_array)
            self.add_input("data:aerodynamics:aircraft:cruise:CD:compressibility", val=nans_array)
            self.add_input("data:aerodynamics:aircraft:cruise:induced_drag_coefficient", val=np.nan)

            self.add_output("data:aerodynamics:aircraft:cruise:CD", shape=POLAR_POINT_COUNT)
            self.add_output("data:aerodynamics:aircraft:cruise:L_D_max")
            self.add_output("data:aerodynamics:aircraft:cruise:optimal_CL")
            self.add_output("data:aerodynamics:aircraft:cruise:optimal_CD")

        self.declare_partials("*", "*", method="fd")

    def compute(self, inputs, outputs):
        k_cd = inputs["tuning:aerodynamics:aircraft:cruise:CD:k"]
        offset_cd = inputs["tuning:aerodynamics:aircraft:cruise:CD:offset"]
        k_winglet_cd = inputs["tuning:aerodynamics:aircraft:cruise:CD:winglet_effect:k"]
        offset_winglet_cd = inputs["tuning:aerodynamics:aircraft:cruise:CD:winglet_effect:offset"]
        if self.low_speed_aero:
            cl = inputs["cl_low_speed"]
            cd0 = inputs["cd0_total_low_speed"]
            cd_trim = inputs["cd_trim_low_speed"]
            cd_c = inputs["cd_comp_low_speed"]
            coef_k = inputs["oswald_coeff_low_speed"]
        else:
            cl = inputs["data:aerodynamics:aircraft:cruise:CL"]
            cd0 = inputs["data:aerodynamics:aircraft:cruise:CD0"]
            cd_trim = inputs["data:aerodynamics:aircraft:cruise:CD:trim"]
            cd_c = inputs["data:aerodynamics:aircraft:cruise:CD:compressibility"]
            coef_k = inputs["data:aerodynamics:aircraft:cruise:induced_drag_coefficient"]

        cd = (
            cd0 + cd_c + cd_trim + coef_k * cl ** 2 * k_winglet_cd + offset_winglet_cd
        ) * k_cd + offset_cd

        if self.low_speed_aero:
            outputs["aerodynamics:Cd_low_speed"] = cd
        else:
            outputs["data:aerodynamics:aircraft:cruise:CD"] = cd

            Cl_opt, Cd_opt = get_optimum_ClCd(np.array([cd, cl]))[0:2]
            outputs["data:aerodynamics:aircraft:cruise:L_D_max"] = Cl_opt / Cd_opt
            outputs["data:aerodynamics:aircraft:cruise:optimal_CL"] = Cl_opt
            outputs["data:aerodynamics:aircraft:cruise:optimal_CD"] = Cd_opt


def get_optimum_ClCd(ClCd):
    lift_drag_ratio = ClCd[1, :] / ClCd[0, :]
    optimum_index = np.argmax(lift_drag_ratio)

    optimum_Cz = ClCd[1][optimum_index]
    optimum_Cd = ClCd[0][optimum_index]

    return optimum_Cz, optimum_Cd



"""
Main components for mass breakdown
"""
#  This file is part of FAST : A framework for rapid Overall Aircraft Design
#  Copyright (C) 2020  ONERA & ISAE-SUPAERO
#  FAST is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <https://www.gnu.org/licenses/>.

import openmdao.api as om

from fastoad.models.options import PAYLOAD_FROM_NPAX
from fastoad.models.weight.mass_breakdown.a_airframe import (
    WingWeight,
    FuselageWeight,
    EmpennageWeight,
    FlightControlsWeight,
    LandingGearWeight,
    PylonsWeight,
    PaintWeight,
)
from fastoad.models.weight.mass_breakdown.b_propulsion import (
    EngineWeight,
    FuelLinesWeight,
    UnconsumablesWeight,
)
from fastoad.models.weight.mass_breakdown.c_systems import (
    PowerSystemsWeight,
    LifeSupportSystemsWeight,
    NavigationSystemsWeight,
    TransmissionSystemsWeight,
    FixedOperationalSystemsWeight,
    FlightKitWeight,
)
from fastoad.models.weight.mass_breakdown.cs25 import Loads
from fastoad.models.weight.mass_breakdown.d_furniture import (
    PassengerSeatsWeight,
    FoodWaterWeight,
    SecurityKitWeight,
    ToiletsWeight,
)
from fastoad.models.weight.mass_breakdown.e_crew import CrewWeight
from fastoad.models.weight.mass_breakdown.payload import ComputePayload
from fastoad.models.weight.mass_breakdown.update_mlw_and_mzfw import UpdateMLWandMZFW


class MassBreakdown(om.Group):
    """
    Computes analytically the mass of each part of the aircraft, and the resulting sum,
    the Overall Weight Empty (OWE).

    Some models depend on MZFW (Max Zero Fuel Weight), MLW (Max Landing Weight) and
    MTOW (Max TakeOff Weight), which depend on OWE.

    This model cycles for having consistent OWE, MZFW and MLW.
    Consistency with MTOW can be achieved by cycling with a model that computes MTOW from OWE,
    which should come from a mission computation that will assess needed block fuel.

    Options:
    - payload_from_npax: If True (default), payload masses will be computed from NPAX, if False
                         design payload mass and maximum payload mass must be provided.
    """

    def initialize(self):
        self.options.declare(PAYLOAD_FROM_NPAX, types=bool, default=True)

    def setup(self):
        if self.options[PAYLOAD_FROM_NPAX]:
            self.add_subsystem("payload", ComputePayload(), promotes=["*"])
        self.add_subsystem("owe", OperatingWeightEmpty(), promotes=["*"])
        self.add_subsystem("update_mzfw_and_mlw", UpdateMLWandMZFW(), promotes=["*"])

        # Solvers setup
        self.nonlinear_solver = om.NonlinearBlockGS()
        self.nonlinear_solver.options["iprint"] = 0
        self.nonlinear_solver.options["maxiter"] = 50

        self.linear_solver = om.LinearBlockGS()
        self.linear_solver.options["iprint"] = 0


class OperatingWeightEmpty(om.Group):
    """ Operating Empty Weight (OEW) estimation

    This group aggregates weight from all components of the aircraft.
    """

    def setup(self):
        # Airframe
        self.add_subsystem("loads", Loads(), promotes=["*"])
        self.add_subsystem("wing_weight", WingWeight(), promotes=["*"])
        self.add_subsystem("fuselage_weight", FuselageWeight(), promotes=["*"])
        self.add_subsystem("empennage_weight", EmpennageWeight(), promotes=["*"])
        self.add_subsystem("flight_controls_weight", FlightControlsWeight(), promotes=["*"])
        self.add_subsystem("landing_gear_weight", LandingGearWeight(), promotes=["*"])
        # Engine have to be computed before pylons
        self.add_subsystem("engines_weight", EngineWeight(), promotes=["*"])
        self.add_subsystem("pylons_weight", PylonsWeight(), promotes=["*"])
        self.add_subsystem("paint_weight", PaintWeight(), promotes=["*"])
        # Propulsion
        self.add_subsystem("fuel_lines_weight", FuelLinesWeight(), promotes=["*"])
        self.add_subsystem("unconsumables_weight", UnconsumablesWeight(), promotes=["*"])
        # Systems
        self.add_subsystem("power_systems_weight", PowerSystemsWeight(), promotes=["*"])
        self.add_subsystem(
            "life_support_systems_weight", LifeSupportSystemsWeight(), promotes=["*"]
        )
        self.add_subsystem("navigation_systems_weight", NavigationSystemsWeight(), promotes=["*"])
        self.add_subsystem(
            "transmission_systems_weight", TransmissionSystemsWeight(), promotes=["*"]
        )
        self.add_subsystem(
            "fixed_operational_systems_weight", FixedOperationalSystemsWeight(), promotes=["*"]
        )
        self.add_subsystem("flight_kit_weight", FlightKitWeight(), promotes=["*"])
        # Cargo and furniture
        self.add_subsystem("passenger_seats_weight", PassengerSeatsWeight(), promotes=["*"])
        self.add_subsystem("food_water_weight", FoodWaterWeight(), promotes=["*"])
        self.add_subsystem("security_kit_weight", SecurityKitWeight(), promotes=["*"])
        self.add_subsystem("toilets_weight", ToiletsWeight(), promotes=["*"])
        # Crew
        self.add_subsystem("crew_weight", CrewWeight(), promotes=["*"])

        # Make additions
        self.add_subsystem(
            "airframe_weight_sum",
            om.AddSubtractComp(
                "data:weight:airframe:mass",
                [
                    "data:weight:airframe:wing:mass",
                    "data:weight:airframe:fuselage:mass",
                    "data:weight:airframe:horizontal_tail:mass",
                    "data:weight:airframe:vertical_tail:mass",
                    "data:weight:airframe:flight_controls:mass",
                    "data:weight:airframe:landing_gear:main:mass",
                    "data:weight:airframe:landing_gear:front:mass",
                    "data:weight:airframe:pylon:mass",
                    "data:weight:airframe:paint:mass",
                ],
                units="kg",
                desc="Mass of airframe",
            ),
            promotes=["*"],
        )

        self.add_subsystem(
            "propulsion_weight_sum",
            om.AddSubtractComp(
                "data:weight:propulsion:mass",
                [
                    "data:weight:propulsion:engine:mass",
                    "data:weight:propulsion:fuel_lines:mass",
                    "data:weight:propulsion:unconsumables:mass",
                ],
                units="kg",
                desc="Mass of the propulsion system",
            ),
            promotes=["*"],
        )

        self.add_subsystem(
            "systems_weight_sum",
            om.AddSubtractComp(
                "data:weight:systems:mass",
                [
                    "data:weight:systems:power:auxiliary_power_unit:mass",
                    "data:weight:systems:power:electric_systems:mass",
                    "data:weight:systems:power:hydraulic_systems:mass",
                    "data:weight:systems:life_support:insulation:mass",
                    "data:weight:systems:life_support:air_conditioning:mass",
                    "data:weight:systems:life_support:de-icing:mass",
                    "data:weight:systems:life_support:cabin_lighting:mass",
                    "data:weight:systems:life_support:seats_crew_accommodation:mass",
                    "data:weight:systems:life_support:oxygen:mass",
                    "data:weight:systems:life_support:safety_equipment:mass",
                    "data:weight:systems:navigation:mass",
                    "data:weight:systems:transmission:mass",
                    "data:weight:systems:operational:radar:mass",
                    "data:weight:systems:operational:cargo_hold:mass",
                    "data:weight:systems:flight_kit:mass",
                ],
                units="kg",
                desc="Mass of aircraft systems",
            ),
            promotes=["*"],
        )

        self.add_subsystem(
            "furniture_weight_sum",
            om.AddSubtractComp(
                "data:weight:furniture:mass",
                [
                    "data:weight:furniture:passenger_seats:mass",
                    "data:weight:furniture:food_water:mass",
                    "data:weight:furniture:security_kit:mass",
                    "data:weight:furniture:toilets:mass",
                ],
                units="kg",
                desc="Mass of aircraft furniture",
            ),
            promotes=["*"],
        )

        self.add_subsystem(
            "OWE_sum",
            om.AddSubtractComp(
                "data:weight:aircraft:OWE",
                [
                    "data:weight:airframe:mass",
                    "data:weight:propulsion:mass",
                    "data:weight:systems:mass",
                    "data:weight:furniture:mass",
                    "data:weight:crew:mass",
                ],
                units="kg",
                desc="Mass of crew",
            ),
            promotes=["*"],
        )



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['fastoad',
 'fastoad.cmd',
 'fastoad.cmd.resources',
 'fastoad.cmd.tests',
 'fastoad.cmd.tests.data.sellar_example',
 'fastoad.io',
 'fastoad.io.configuration',
 'fastoad.io.configuration.tests',
 'fastoad.io.configuration.tests.data.sellar_example',
 'fastoad.io.xml',
 'fastoad.io.xml.resources',
 'fastoad.io.xml.tests',
 'fastoad.models',
 'fastoad.models.aerodynamics',
 'fastoad.models.aerodynamics.components',
 'fastoad.models.aerodynamics.components.resources',
 'fastoad.models.aerodynamics.components.tests',
 'fastoad.models.aerodynamics.external',
 'fastoad.models.aerodynamics.external.xfoil',
 'fastoad.models.aerodynamics.external.xfoil.resources',
 'fastoad.models.aerodynamics.external.xfoil.tests',
 'fastoad.models.aerodynamics.external.xfoil.xfoil699',
 'fastoad.models.aerodynamics.tests',
 'fastoad.models.geometry',
 'fastoad.models.geometry.geom_components',
 'fastoad.models.geometry.geom_components.fuselage',
 'fastoad.models.geometry.geom_components.ht',
 'fastoad.models.geometry.geom_components.ht.components',
 'fastoad.models.geometry.geom_components.nacelle_pylons',
 'fastoad.models.geometry.geom_components.vt',
 'fastoad.models.geometry.geom_components.vt.components',
 'fastoad.models.geometry.geom_components.wing',
 'fastoad.models.geometry.geom_components.wing.components',
 'fastoad.models.geometry.profiles',
 'fastoad.models.geometry.resources',
 'fastoad.models.geometry.tests',
 'fastoad.models.handling_qualities',
 'fastoad.models.handling_qualities.tail_sizing',
 'fastoad.models.handling_qualities.tests',
 'fastoad.models.loops',
 'fastoad.models.loops.tests',
 'fastoad.models.performances',
 'fastoad.models.performances.tests',
 'fastoad.models.propulsion',
 'fastoad.models.propulsion.fuel_engine',
 'fastoad.models.propulsion.fuel_engine.rubber_engine',
 'fastoad.models.propulsion.fuel_engine.rubber_engine.tests',
 'fastoad.models.weight',
 'fastoad.models.weight.cg',
 'fastoad.models.weight.cg.cg_components',
 'fastoad.models.weight.cg.tests',
 'fastoad.models.weight.mass_breakdown',
 'fastoad.models.weight.mass_breakdown.a_airframe',
 'fastoad.models.weight.mass_breakdown.b_propulsion',
 'fastoad.models.weight.mass_breakdown.c_systems',
 'fastoad.models.weight.mass_breakdown.d_furniture',
 'fastoad.models.weight.mass_breakdown.e_crew',
 'fastoad.models.weight.mass_breakdown.tests',
 'fastoad.module_management',
 'fastoad.module_management.tests',
 'fastoad.module_management.tests.dummy_pelix_bundles',
 'fastoad.module_management.tests.sellar_example',
 'fastoad.notebooks',
 'fastoad.notebooks.tutorial',
 'fastoad.notebooks.tutorial.data',
 'fastoad.notebooks.tutorial.img',
 'fastoad.openmdao',
 'fastoad.openmdao.resources',
 'fastoad.openmdao.tests',
 'fastoad.openmdao.tests.sellar_example',
 'fastoad.utils',
 'fastoad.utils.physics',
 'fastoad.utils.physics.tests',
 'fastoad.utils.postprocessing',
 'fastoad.utils.postprocessing.tests',
 'fastoad.utils.postprocessing.tests.data.sellar_example',
 'fastoad.utils.resource_management',
 'fastoad.utils.tests']

package_data = \
{'': ['*'],
 'fastoad.cmd.tests': ['data/*'],
 'fastoad.io.configuration.tests': ['data/*'],
 'fastoad.io.xml.tests': ['data/*'],
 'fastoad.models.aerodynamics.components.tests': ['data/*'],
 'fastoad.models.aerodynamics.tests': ['data/*'],
 'fastoad.models.geometry.tests': ['data/*'],
 'fastoad.models.handling_qualities.tests': ['data/*'],
 'fastoad.models.weight.cg.tests': ['data/*'],
 'fastoad.models.weight.mass_breakdown.tests': ['data/*'],
 'fastoad.utils.postprocessing.tests': ['data/*']}

install_requires = \
['importlib_resources',
 'ipopo>=1,<2',
 'ipysheet',
 'ipywidgets',
 'jupyterlab',
 'lxml',
 'matplotlib',
 'numpy',
 'openmdao>=3,<4',
 'pandas>=1,<2',
 'plotly',
 'scipy',
 'tomlkit>=0.6.0,<0.7.0',
 'wop>=1.8.0,<2.0.0']

entry_points = \
{'console_scripts': ['fastoad = fastoad.cmd.fast:main']}

setup_kwargs = {
    'name': 'fast-oad',
    'version': '0.2.0b0',
    'description': 'FAST-OAD is a framework for performing rapid Overall Aircraft Design',
    'long_description': '![Tests](https://github.com/fast-aircraft-design/FAST-OAD/workflows/Tests/badge.svg)\n[![Documentation Status](https://readthedocs.org/projects/fast-oad/badge/?version=latest)](https://fast-oad.readthedocs.io/en/latest/?badge=latest)\n[![Codacy Badge](https://app.codacy.com/project/badge/Grade/e0f42000b8af4ee999dbdcc80eeabfdc)](https://www.codacy.com/gh/fast-aircraft-design/FAST-OAD?utm_source=github.com&amp;utm_medium=referral&amp;utm_content=fast-aircraft-design/FAST-OAD&amp;utm_campaign=Badge_Grade)\n[![codecov](https://codecov.io/gh/fast-aircraft-design/FAST-OAD/branch/master/graph/badge.svg)](https://codecov.io/gh/fast-aircraft-design/FAST-OAD)\n\nFAST-OAD: Future Aircraft Sizing Tool - Overall Aircraft Design\n===============================================================\n\nFAST-OAD is a framework for performing rapid Overall Aircraft Design.\n\nIt proposes multi-disciplinary analysis and optimisation by relying on\nthe [OpenMDAO framework](https://openmdao.org/).\n\nFAST-OAD allows easy switching between models for a same discipline, and\nalso adding/removing disciplines to match the need of your study.\n\nCurrently, FAST-OAD is bundled with models for commercial transport\naircraft of years 1990-2000. Other models will come and you may create\nyour own models and use them instead of bundled ones.\n\nMore details can be found in the [official\ndocumentation](https://fast-oad.readthedocs.io/).\n\nInstall\n-------\n\n**Prerequisite**:FAST-OAD needs at least **Python 3.6.1** (usage of\n**Python 3.8.**\\* is discouraged on Windows: some additional features of\n FAST-OAD require Jupyter notebooks, which are for now\n [not\ncompatible with\nit](https://github.com/jupyterlab/jupyterlab/issues/6487)).\n\nIt is recommended (but not required) to install FAST-OAD in a virtual\nenvironment ([conda](https://docs.conda.io/en/latest/),\n[venv](https://docs.python.org/3.7/library/venv.html), ...)\n\nOnce Python is installed, FAST-OAD can be installed using pip.\n\n> **Note**: If your network uses a proxy, you may have to do [some\n> settings](https://pip.pypa.io/en/stable/user_guide/#using-a-proxy-server)\n> for pip to work correctly\n\nYou can install the latest version with this command:\n\n``` {.bash}\n$ pip install --upgrade FAST-OAD\n```\n',
    'author': 'Christophe DAVID',
    'author_email': 'christophe.david@onera.fr',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/fast-aircraft-design/FAST-OAD',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6.1,<4.0.0',
}


setup(**setup_kwargs)



"""
    Estimation of total aircraft wet area
"""

#  This file is part of FAST : A framework for rapid Overall Aircraft Design
#  Copyright (C) 2020  ONERA & ISAE-SUPAERO
#  FAST is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <https://www.gnu.org/licenses/>.
import numpy as np
from openmdao.core.explicitcomponent import ExplicitComponent


class ComputeTotalArea(ExplicitComponent):
    # TODO: Document equations. Cite sources
    """ Total aircraft wet area estimation """

    def setup(self):
        self.add_input("data:geometry:wing:wetted_area", val=np.nan, units="m**2")
        self.add_input("data:geometry:fuselage:wetted_area", val=np.nan, units="m**2")
        self.add_input("data:geometry:horizontal_tail:wetted_area", val=np.nan, units="m**2")
        self.add_input("data:geometry:vertical_tail:wetted_area", val=np.nan, units="m**2")
        self.add_input("data:geometry:propulsion:nacelle:wetted_area", val=np.nan, units="m**2")
        self.add_input("data:geometry:propulsion:pylon:wetted_area", val=np.nan, units="m**2")
        self.add_input("data:geometry:propulsion:engine:count", val=np.nan)

        self.add_output("data:geometry:aircraft:wetted_area", units="m**2")

        self.declare_partials("data:geometry:aircraft:wetted_area", "*", method="fd")

    def compute(self, inputs, outputs):
        wet_area_wing = inputs["data:geometry:wing:wetted_area"]
        wet_area_fus = inputs["data:geometry:fuselage:wetted_area"]
        wet_area_ht = inputs["data:geometry:horizontal_tail:wetted_area"]
        wet_area_vt = inputs["data:geometry:vertical_tail:wetted_area"]
        wet_area_nac = inputs["data:geometry:propulsion:nacelle:wetted_area"]
        wet_area_pylon = inputs["data:geometry:propulsion:pylon:wetted_area"]
        n_engines = inputs["data:geometry:propulsion:engine:count"]

        wet_area_total = (
            wet_area_wing
            + wet_area_fus
            + wet_area_ht
            + wet_area_vt
            + n_engines * (wet_area_nac + wet_area_pylon)
        )

        outputs["data:geometry:aircraft:wetted_area"] = wet_area_total



"""
    Estimation of wing lift coefficient
"""

#  This file is part of FAST : A framework for rapid Overall Aircraft Design
#  Copyright (C) 2020  ONERA & ISAE-SUPAERO
#  FAST is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <https://www.gnu.org/licenses/>.
import math

import numpy as np
from openmdao.core.explicitcomponent import ExplicitComponent


# TODO: This belongs more to aerodynamics than geometry
class ComputeCLalpha(ExplicitComponent):
    # TODO: Document equations. Cite sources
    """ Wing lift coefficient estimation """

    def setup(self):
        self.add_input("data:TLAR:cruise_mach", val=np.nan)
        self.add_input("data:geometry:fuselage:maximum_width", val=np.nan, units="m")
        self.add_input("data:geometry:fuselage:maximum_height", val=np.nan, units="m")
        self.add_input("data:geometry:wing:area", val=np.nan, units="m**2")
        self.add_input("data:geometry:wing:root:chord", val=np.nan, units="m")
        self.add_input("data:geometry:wing:tip:chord", val=np.nan, units="m")
        self.add_input("data:geometry:wing:tip:thickness_ratio", val=np.nan)
        self.add_input("data:geometry:wing:sweep_25", val=np.nan, units="deg")
        self.add_input("data:geometry:wing:aspect_ratio", val=np.nan)
        self.add_input("data:geometry:wing:span", val=np.nan, units="m")

        self.add_output("data:aerodynamics:aircraft:cruise:CL_alpha")

        self.declare_partials("data:aerodynamics:aircraft:cruise:CL_alpha", "*", method="fd")

    def compute(self, inputs, outputs):
        cruise_mach = inputs["data:TLAR:cruise_mach"]
        width_max = inputs["data:geometry:fuselage:maximum_width"]
        height_max = inputs["data:geometry:fuselage:maximum_height"]
        span = inputs["data:geometry:wing:span"]
        lambda_wing = inputs["data:geometry:wing:aspect_ratio"]
        el_ext = inputs["data:geometry:wing:tip:thickness_ratio"]
        wing_area = inputs["data:geometry:wing:area"]
        l2_wing = inputs["data:geometry:wing:root:chord"]
        l4_wing = inputs["data:geometry:wing:tip:chord"]
        sweep_25 = inputs["data:geometry:wing:sweep_25"]

        beta = math.sqrt(1 - cruise_mach ** 2)
        d_f = math.sqrt(width_max * height_max)
        fact_f = 1.07 * (1 + d_f / span) ** 2
        lambda_wing_eff = lambda_wing * (1 + 1.9 * l4_wing * el_ext / span)
        cl_alpha_wing = (
            2
            * math.pi
            * lambda_wing_eff
            / (
                2
                + math.sqrt(
                    4
                    + lambda_wing_eff ** 2
                    * beta ** 2
                    / 0.95 ** 2
                    * (1 + (math.tan(sweep_25 / 180.0 * math.pi)) ** 2 / beta ** 2)
                )
            )
            * (wing_area - l2_wing * width_max)
            / wing_area
            * fact_f
        )

        outputs["data:aerodynamics:aircraft:cruise:CL_alpha"] = cl_alpha_wing
